@@ -1,6 +1,6 @@
 ﻿/***********************************************************************
  *
- * Copyright (C) 2014-2021 wereturtle
+ * Copyright (C) 2014-2022 wereturtle
  * Copyright (C) 2009, 2010, 2011, 2012, 2013, 2014 Graeme Gott <graeme@gottcode.org>
  *
  * This program is free software: you can redistribute it and/or modify
@@ -18,8 +18,6 @@
  *
  ***********************************************************************/
 
-#include <QDebug>
-
 #include <QApplication>
 #include <QClipboard>
 #include <QCommonStyle>
@@ -38,6 +36,7 @@
 #include <QMenuBar>
 #include <QScrollBar>
 #include <QSettings>
+#include <QSizePolicy>
 #include <QStatusBar>
 #include <QTemporaryFile>
 
@@ -71,10 +70,12 @@ enum SidebarTabIndex {
 
 #define GW_MAIN_WINDOW_GEOMETRY_KEY "Window/mainWindowGeometry"
 #define GW_MAIN_WINDOW_STATE_KEY "Window/mainWindowState"
+#define GW_SPLITTER_GEOMETRY_KEY "Window/splitterGeometry"
 
 MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     : QMainWindow(parent)
 {
+    this->focusModeEnabled = false;
     this->awesome = new QtAwesome(qApp);
     this->awesome->initFontAwesome();
     QString fileToOpen;
@@ -92,6 +93,7 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     MarkdownDocument *document = new MarkdownDocument();
 
     editor = new MarkdownEditor(document, theme.lightColorScheme(), this);
+    editor->setMinimumWidth(0.1 * qApp->primaryScreen()->size().width());
     editor->setFont(appSettings->editorFont().family(), appSettings->editorFont().pointSize());
     editor->setUseUnderlineForEmphasis(appSettings->useUnderlineForEmphasis());
     editor->setEnableLargeHeadingSizes(appSettings->largeHeadingSizesEnabled());
@@ -309,38 +311,52 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     connect(outlineWidget, SIGNAL(headingNumberNavigated(int)), htmlPreview, SLOT(navigateToHeading(int)));
     connect(appSettings, SIGNAL(currentHtmlExporterChanged(Exporter *)), htmlPreview, SLOT(setHtmlExporter(Exporter *)));
 
-    htmlPreview->setMinimumWidth(0);
+    htmlPreview->setMinimumWidth(0.1 * qApp->primaryScreen()->size().width());
     htmlPreview->setObjectName("htmlpreview");
     htmlPreview->setVisible(appSettings->htmlPreviewVisible());
 
-    previewSplitter = new QSplitter(this);
-    previewSplitter->addWidget(editorPane);
-    previewSplitter->addWidget(htmlPreview);
-    previewSplitter->setCollapsible(0, true);
-    previewSplitter->setCollapsible(1, true);
-
-
     this->findReplace = new FindReplace(this->editor, this);
     statusBarWidgets.append(this->findReplace);
     this->findReplace->setVisible(false);
 
     buildMenuBar();
     buildStatusBar();
-    
-    QVBoxLayout *mainLayout = new QVBoxLayout();
-    QWidget *mainPane = new QWidget(this);
-    mainPane->setLayout(mainLayout);
-    mainLayout->addWidget(previewSplitter, 500);
-    mainLayout->setSpacing(0);
-    mainLayout->setMargin(0);
 
-    sidebarSplitter = new QSplitter(this);
-    sidebarSplitter->addWidget(sidebar);
-    sidebarSplitter->addWidget(mainPane);
-    sidebarSplitter->setCollapsible(0, false);
-    sidebarSplitter->setCollapsible(1, true);
+    sidebar->setMinimumWidth(0.1 * qApp->primaryScreen()->size().width());
 
-    this->setCentralWidget(sidebarSplitter);
+    splitter = new QSplitter(this);
+    splitter->addWidget(sidebar);
+    splitter->addWidget(editorPane);
+    splitter->addWidget(htmlPreview);
+    splitter->setChildrenCollapsible(false);
+    splitter->setStretchFactor(0, 0);
+    splitter->setStretchFactor(1, 2);
+    splitter->setStretchFactor(2, 1);
+
+    // Set default sizes for splitter.
+    QList<int> sizes;
+    int sidebarWidth = this->width() * 0.2;
+    int otherWidth = (this->width() - sidebarWidth) / 2;
+    sizes.append(sidebarWidth);
+    sizes.append(otherWidth);
+    sizes.append(otherWidth);
+
+    splitter->setSizes(sizes);
+
+    // If previous splitter geometry was stored, load it.
+    if (windowSettings.contains(GW_SPLITTER_GEOMETRY_KEY)) {
+        this->splitter->restoreState(windowSettings.value(GW_SPLITTER_GEOMETRY_KEY).toByteArray());
+    }
+
+    this->connect(splitter,
+        &QSplitter::splitterMoved,
+        [this](int pos, int index) {
+            Q_UNUSED(pos)
+            Q_UNUSED(index)
+            adjustEditor();
+        });
+
+    this->setCentralWidget(splitter);
 
     // Show the main window.
     show();
@@ -350,7 +366,7 @@ MainWindow::MainWindow(const QString &filePath, QWidget *parent)
     // the theme is applied before show().
     //
     applyTheme();
-    adjustEditorWidth(this->width());
+    adjustEditor();
 
     this->update();
     qApp->processEvents();
@@ -386,7 +402,27 @@ QSize MainWindow::sizeHint() const
 
 void MainWindow::resizeEvent(QResizeEvent *event)
 {
-    adjustEditorWidth(event->size().width());
+    int width = event->size().width();
+
+    if (width < (0.5 * qApp->primaryScreen()->size().width())) {
+        this->sidebar->setVisible(false);
+        this->sidebar->setAutoHideEnabled(true);
+        this->sidebarHiddenForResize = true;
+    }
+    else {
+        this->sidebarHiddenForResize = false;
+
+        if (!this->focusModeEnabled && this->appSettings->sidebarVisible()) {
+            this->sidebar->setAutoHideEnabled(false);
+            this->sidebar->setVisible(true);
+        }
+        else {
+            this->sidebar->setAutoHideEnabled(true);
+            this->sidebar->setVisible(false);
+        }
+    }
+
+    adjustEditor();
 }
 
 void MainWindow::keyPressEvent(QKeyEvent *e)
@@ -465,6 +501,7 @@ void MainWindow::quitApplication()
 
         windowSettings.setValue(GW_MAIN_WINDOW_GEOMETRY_KEY, saveGeometry());
         windowSettings.setValue(GW_MAIN_WINDOW_STATE_KEY, saveState());
+        windowSettings.setValue(GW_SPLITTER_GEOMETRY_KEY, splitter->saveState());
         windowSettings.sync();
 
         DictionaryManager::instance().addProviders();
@@ -506,17 +543,16 @@ void MainWindow::openPreferencesDialog()
 
 void MainWindow::toggleHtmlPreview(bool checked)
 {
-    QList<int> splitterSizes;
-
     htmlPreviewMenuAction->blockSignals(true);
 
     htmlPreviewMenuAction->setChecked(checked);
     htmlPreview->setVisible(checked);
     htmlPreview->updatePreview();
-    adjustEditorWidth(this->width());
 
     htmlPreviewMenuAction->blockSignals(false);
     appSettings->setHtmlPreviewVisible(checked);
+    this->update();
+    adjustEditor();
 }
 
 void MainWindow::toggleHemingwayMode(bool checked)
@@ -530,13 +566,20 @@ void MainWindow::toggleHemingwayMode(bool checked)
 
 void MainWindow::toggleFocusMode(bool checked)
 {
+    this->focusModeEnabled = checked;
+
     if (checked) {
-        editor->setFocusMode(appSettings->focusMode());        
+        editor->setFocusMode(appSettings->focusMode());
+        sidebar->setVisible(false);
+        sidebar->setAutoHideEnabled(true);
     } else {
         editor->setFocusMode(FocusModeDisabled);
-    }
 
-    sidebar->setAutoHideEnabled(checked);
+        if (!this->sidebarHiddenForResize && this->appSettings->sidebarVisible()) {
+            sidebar->setAutoHideEnabled(false);
+            sidebar->setVisible(true);
+        }
+    }
 }
 
 void MainWindow::toggleFullScreen(bool checked)
@@ -625,7 +668,7 @@ void MainWindow::toggleDisplayTimeInFullScreen(bool checked)
 void MainWindow::changeEditorWidth(EditorWidth editorWidth)
 {
     editor->setEditorWidth(editorWidth);
-    adjustEditorWidth(this->width());
+    adjustEditor();
 }
 
 void MainWindow::changeInterfaceStyle(InterfaceStyle style)
@@ -694,7 +737,7 @@ void MainWindow::showAbout()
     QString aboutText =
         QString("<p><b>") +  qAppName() + QString(" ")
         + qApp->applicationVersion() + QString("</b></p>")
-        + tr("<p>Copyright &copy; 2014-2021 wereturtle</b>"
+        + tr("<p>Copyright &copy; 2014-2022 wereturtle</b>"
              "<p>You may use and redistribute this software under the terms of the "
              "<a href=\"http://www.gnu.org/licenses/gpl.html\">"
              "GNU General Public License Version 3</a>.</p>"
@@ -897,8 +940,6 @@ void MainWindow::onAboutToShowMenuBarMenu()
 
 void MainWindow::onSidebarVisibilityChanged(bool visible)
 {
-    this->adjustEditorWidth(this->width());
-
     if (visible) {
         toggleSidebarButton->setText(QChar(fa::chevronleft));
     } else {
@@ -908,17 +949,30 @@ void MainWindow::onSidebarVisibilityChanged(bool visible)
     if (!visible) {
         editor->setFocus();
     }
+
+    this->showSidebarAction->blockSignals(true);
+    this->showSidebarAction->setChecked(visible);
+    this->showSidebarAction->blockSignals(false);
+
+    this->adjustEditor();
 }
 
 void MainWindow::toggleSidebarVisible(bool visible)
 {
     this->appSettings->setSidebarVisible(visible);
-    this->sidebar->setAutoHideEnabled(!visible);
-    this->sidebar->setVisible(visible);
 
-    this->showSidebarAction->blockSignals(true);
-    this->showSidebarAction->setChecked(visible);
-    this->showSidebarAction->blockSignals(false);
+    if (!this->sidebarHiddenForResize
+            && !this->focusModeEnabled
+            && this->appSettings->sidebarVisible()) {
+        sidebar->setAutoHideEnabled(false);
+    }
+    else {
+        sidebar->setAutoHideEnabled(true);
+    }
+
+    this->sidebar->setVisible(visible);
+    this->sidebar->setFocus();
+    adjustEditor();
 }
 
 QAction* MainWindow::createWindowAction
@@ -1311,8 +1365,9 @@ void MainWindow::buildSidebar()
     
     connect(this->showSidebarAction,
         &QAction::toggled,
-        this,
-        &MainWindow::toggleSidebarVisible);
+        [this](bool visible) {
+            this->toggleSidebarVisible(visible);
+        });
 
     cheatSheetWidget = new QListWidget(this);
 
@@ -1384,8 +1439,9 @@ void MainWindow::buildSidebar()
     connect(editor, SIGNAL(typingResumed()), sessionStats, SLOT(onTypingResumed()));
 
     sidebar = new Sidebar(this);
-    sidebar->setMinimumWidth(0.10 * QGuiApplication::primaryScreen()->availableSize().width());
-    sidebar->setMaximumWidth(0.33 * QGuiApplication::primaryScreen()->availableSize().width());
+    sidebar->setSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
+    sidebar->setMinimumWidth(0.1 * QGuiApplication::primaryScreen()->availableSize().width());
+    sidebar->setMaximumWidth(0.5 * QGuiApplication::primaryScreen()->availableSize().width());
 
     QPushButton *tabButton = new QPushButton();
     tabButton->setFont(this->awesome->font(style::stfas, 16));
@@ -1459,42 +1515,41 @@ void MainWindow::buildSidebar()
         this,
         &MainWindow::onSidebarVisibilityChanged);
     
-    this->toggleSidebarVisible(appSettings->sidebarVisible());
+    if (!this->sidebarHiddenForResize
+            && !this->focusModeEnabled
+            && appSettings->sidebarVisible()) {
+        this->sidebar->setAutoHideEnabled(false);
+        this->sidebar->setVisible(true);
+    }
+    else {
+        this->sidebar->setAutoHideEnabled(true);
+        this->sidebar->setVisible(false);
+    }
 }
 
-void MainWindow::adjustEditorWidth(int width)
+void MainWindow::adjustEditor()
 {
-    QList<int> sidebarSplitterSizes;
-    QList<int> previewSplitterSizes;
-    int editorWidth = width;
+    // Make sure editor size is updated.
+    qApp->processEvents();
 
-    if (width < (0.5 * qApp->primaryScreen()->size().width())) {
-        sidebar->setMinimumWidth(0);
-        sidebarSplitterSizes.append(0);
-    } else {
-        sidebar->setMinimumWidth(0.1 * qApp->primaryScreen()->size().width());
-        sidebar->setMaximumWidth(0.5 * this->width());
-        sidebar->resize(sidebar->sizeHint().width(), this->height());
-        sidebarSplitterSizes.append(sidebar->width());
-        editorWidth -= sidebar->width();
+    int width = this->width();
+    int sidebarWidth = 0;
+
+    // Make sure live preview does not crowd out editor.
+    // It should not take up more than 50% of the window space
+    // left after the sidebar is accounted for.
+    //
+    if (this->sidebar->isVisible()) {
+        sidebarWidth = this->sidebar->width();
     }
 
-    sidebarSplitterSizes.append(editorWidth);
+    this->htmlPreview->setMaximumWidth((width - sidebarWidth) / 2);
 
-    if (htmlPreview->isVisible()) {
-        editorWidth /= 2;
-        previewSplitterSizes.append(editorWidth);
-    }
-
-    previewSplitterSizes.append(editorWidth);
-    previewSplitter->setSizes(previewSplitterSizes);
-    sidebarSplitter->setSizes(sidebarSplitterSizes);
-
-    // Resize the editor's margins based on the size of the window.
-    editor->setupPaperMargins();
+    // Resize the editor's margins.
+    this->editor->setupPaperMargins();
 
     // Scroll to cursor position.
-    editor->centerCursor();
+    this->editor->centerCursor();
 }
 
 void MainWindow::applyTheme()
@@ -1523,8 +1578,7 @@ void MainWindow::applyTheme()
     //
     qApp->setStyleSheet(styler.layoutStyleSheet());
 
-    previewSplitter->setStyleSheet(styler.splitterStyleSheet());
-    sidebarSplitter->setStyleSheet(styler.splitterStyleSheet());
+    this->splitter->setStyleSheet(styler.splitterStyleSheet());
     this->statusBar()->setStyleSheet(styler.statusBarStyleSheet());
 
     foreach (QWidget *w, statusBarWidgets) {
@@ -1548,7 +1602,7 @@ void MainWindow::applyTheme()
 
     htmlPreview->setStyleSheet(styler.htmlPreviewCss());
 
-    adjustEditorWidth(this->width());
+    adjustEditor();
 }
 
 } // namespace ghostwriter
