@@ -1,6 +1,6 @@
 ﻿/***********************************************************************
  *
- * Copyright (C) 2014-2021 wereturtle
+ * Copyright (C) 2014-2022 wereturtle
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -104,7 +104,7 @@ private:
     MarkdownEditor *editor;
     FindReplace* findReplace;
     QSplitter *previewSplitter;
-    QSplitter *sidebarSplitter;
+    QSplitter *splitter;
     DocumentManager *documentManager;
     ThemeRepository *themeRepo;
     Theme theme;
@@ -134,6 +134,8 @@ private:
     QAction *recentFilesActions[MAX_RECENT_FILES];
     bool menuBarMenuActivated;
     QAction *showSidebarAction;
+    bool sidebarHiddenForResize;
+    bool focusModeEnabled;
 
     QList<QWidget *> statusBarButtons;
     QList<QWidget *> statusBarWidgets;
@@ -160,7 +162,7 @@ private:
     void buildStatusBar();
     void buildSidebar();
 
-    void adjustEditorWidth(int width);
+    void adjustEditor();
 };
 } // namespace ghostwriter
 
